"""Tests for the CapabilityRegistry class."""

import json
from pathlib import Path
from unittest.mock import Mock, patch

import httpx
import pytest
from llm_registry import CapabilityRegistry, Provider


@pytest.fixture
def sample_models_json(tmp_path):
    """Create a sample models.json file."""
    models_data = {
        "version": "1.0.0",
        "last_updated": "2024-03-02",
        "models": {
            "o3-mini": {
                "providers": ["openai"],
                "model_family": "o3",
                "api_params": {
                    "frequency_penalty": True,
                    "max_tokens": True,
                    "n": True,
                    "presence_penalty": True,
                    "temperature": True,
                    "top_p": True
                },
                "features": {
                    "streaming": True,
                    "tools": True,
                    "vision": False,
                    "json_mode": True,
                    "system_prompt": True
                },
                "token_costs": {
                    "input_cost": 0.2,
                    "output_cost": 0.4,
                    "context_window": 200000,
                    "training_cutoff": "2024-01"
                }
            }
        }
    }

    json_file = tmp_path / "models.json"
    with open(json_file, "w") as f:
        json.dump(models_data, f)
    return json_file


def test_get_models_offline(sample_models_json):
    """Test getting models in offline mode."""
    with patch("llm_registry.registry.CapabilityRegistry._get_packaged_data") as mock_get_data:
        mock_get_data.return_value = json.loads(sample_models_json.read_text())
        registry = CapabilityRegistry(offline_mode=True)
        models = registry.get_models()

        assert len(models) == 1
        model = models[0]
        assert model.model_id == "o3-mini"
        assert Provider.OPENAI in model.providers
        assert model.features.vision is False
        assert model.token_costs.input_cost == 0.2


def test_get_models_by_provider(sample_models_json):
    """Test filtering models by provider."""
    with patch("llm_registry.registry.CapabilityRegistry._get_packaged_data") as mock_get_data:
        mock_get_data.return_value = json.loads(sample_models_json.read_text())
        registry = CapabilityRegistry(offline_mode=True)

        # Should find the OpenAI model
        models = registry.get_models(provider=Provider.OPENAI)
        assert len(models) == 1

        # Should not find any Anthropic models
        models = registry.get_models(provider=Provider.ANTHROPIC)
        assert len(models) == 0


def test_get_specific_model(sample_models_json):
    """Test getting a specific model by ID."""
    with patch("llm_registry.registry.CapabilityRegistry._get_packaged_data") as mock_get_data:
        mock_get_data.return_value = json.loads(sample_models_json.read_text())
        registry = CapabilityRegistry(offline_mode=True)

        # Should find existing model
        model = registry.get_model(Provider.OPENAI, "o3-mini")
        assert model is not None
        assert model.model_id == "o3-mini"

        # Should return None for non-existent model
        model = registry.get_model(Provider.OPENAI, "non-existent")
        assert model is None


@pytest.mark.asyncio
async def test_remote_fallback(sample_models_json):
    """Test fallback to packaged data when remote fails."""
    with patch("llm_registry.registry.CapabilityRegistry._get_packaged_data") as mock_get_data:
        mock_get_data.return_value = json.loads(sample_models_json.read_text())

        # Mock httpx to simulate remote failure
        with patch("httpx.Client") as mock_client:
            mock_response = Mock()
            mock_response.raise_for_status.side_effect = httpx.HTTPError("Failed")
            mock_client.return_value.get.return_value = mock_response

            registry = CapabilityRegistry(offline_mode=False)
            models = registry.get_models()

            # Should fall back to packaged data
            assert len(models) == 1
            assert models[0].model_id == "o3-mini"


def test_cache_behavior(sample_models_json):
    """Test caching of remote data."""
    with patch("llm_registry.registry.CapabilityRegistry._get_packaged_data") as mock_get_data:
        mock_get_data.return_value = json.loads(sample_models_json.read_text())

        # Mock httpx to count calls
        with patch("httpx.Client") as mock_client:
            mock_response = Mock()
            mock_response.raise_for_status.return_value = None
            mock_response.json.return_value = json.loads(sample_models_json.read_text())
            mock_client.return_value.get.return_value = mock_response

            registry = CapabilityRegistry(offline_mode=False, cache_ttl=10)

            # First call should hit remote
            registry.get_models()
            assert mock_client.return_value.get.call_count == 1

            # Second call should use cache
            registry.get_models()
            assert mock_client.return_value.get.call_count == 1



"""
CLI interface for LLM Registry.
"""

import json
from functools import lru_cache
from pathlib import Path

import typer
from rich.console import Console
from rich.table import Table

from llm_registry import CapabilityRegistry, Provider
from llm_registry.models import ApiParams, Features, ModelCapabilities, TokenCost

app = typer.Typer(help="LLM Registry CLI")
console = Console()


@lru_cache()
def get_models_file() -> Path:
    """Get the path to models.json file."""
    return Path(__file__).parent / "data" / "models.json"


@lru_cache()
def load_models() -> dict:
    """Load models from JSON file with caching."""
    with open(get_models_file()) as f:
        return json.load(f)


def save_models(data: dict) -> None:
    """Save models to JSON file and invalidate cache."""
    models_file = get_models_file()
    with open(models_file, "w") as f:
        json.dump(data, f, indent=2)
    # Invalidate cache
    load_models.cache_clear()


def get_model_data(model_id: str, data: dict | None = None) -> tuple[dict, dict]:
    """Get model data and validate it exists."""
    if data is None:
        data = load_models()

    if model_id not in data["models"]:
        console.print(f"[red]Error: Model '{model_id}' not found in registry.[/red]")
        raise typer.Exit(code=1)

    return data, data["models"][model_id]


def validate_provider(model_data: dict, provider: Provider, model_id: str) -> None:
    """Validate provider exists for model."""
    if provider and provider.value not in model_data["providers"]:
        console.print(f"[red]Error: Provider '{provider.value}' not found for model '{model_id}'.[/red]")
        raise typer.Exit(code=1)


@app.command()
def list(
    provider: Provider | None = typer.Option(None, help="Filter by provider"),
):
    """
    List available models.
    """
    registry = CapabilityRegistry(offline_mode=True)
    models = registry.get_models(provider=provider)

    if not models:
        console.print("No models found")
        return

    # Create table with minimal formatting
    table = Table(title="LLM Model Capabilities", show_lines=True, header_style="bold", show_header=True, expand=False)
    table.add_column("Model ID", min_width=20, no_wrap=True)
    table.add_column("Providers", min_width=15, no_wrap=True)
    table.add_column("Family", min_width=10, no_wrap=True)
    table.add_column("Input $/M", min_width=12, no_wrap=True)
    table.add_column("Output $/M", min_width=12, no_wrap=True)
    table.add_column("Cache In $/M", min_width=12, no_wrap=True)
    table.add_column("Cache Out $/M", min_width=12, no_wrap=True)
    table.add_column("Context", min_width=8, no_wrap=True)
    table.add_column("Training", min_width=10, no_wrap=True)
    table.add_column("Streaming", min_width=9, justify="center", no_wrap=True)
    table.add_column("Tools", min_width=7, justify="center", no_wrap=True)
    table.add_column("Vision", min_width=7, justify="center", no_wrap=True)
    table.add_column("JSON", min_width=7, justify="center", no_wrap=True)
    table.add_column("System", min_width=7, justify="center", no_wrap=True)

    # Sort models by ID
    models.sort(key=lambda x: x.model_id)

    for model in models:
        providers_str = ", ".join(p.value for p in model.providers)
        table.add_row(
            model.model_id,
            providers_str,
            model.model_family or "",
            f"${model.token_costs.input_cost}" if model.token_costs else "N/A",
            f"${model.token_costs.output_cost}" if model.token_costs else "N/A",
            f"${model.token_costs.cache_input_cost}"
            if model.token_costs and model.token_costs.cache_input_cost is not None
            else "N/A",
            f"${model.token_costs.cache_output_cost}"
            if model.token_costs and model.token_costs.cache_output_cost is not None
            else "N/A",
            str(model.token_costs.context_window) if model.token_costs and model.token_costs.context_window else "N/A",
            model.token_costs.training_cutoff if model.token_costs and model.token_costs.training_cutoff else "N/A",
            "✅" if model.api_params.stream else "❌",
            "✅" if model.features.tools else "❌",
            "✅" if model.features.vision else "❌",
            "✅" if model.features.json_mode else "❌",
            "✅" if model.features.system_prompt else "❌",
        )

    # Print table with full width
    console.print(table, width=200)


@app.command()
def add(
    model_id: str = typer.Argument(..., help="Model identifier (e.g., 'gpt-4')"),
    provider: Provider = typer.Option(..., help="Model provider"),
    model_family: str | None = typer.Option(None, help="Model family (e.g., 'GPT-4')"),
    input_cost: float | None = typer.Option(None, help="Input token cost per 1M tokens"),
    output_cost: float | None = typer.Option(None, help="Output token cost per 1M tokens"),
    context_window: int | None = typer.Option(None, help="Context window size in tokens"),
    training_cutoff: str | None = typer.Option(None, help="Training data cutoff date (e.g., '2024-01')"),
    # API Parameters
    max_tokens: bool = typer.Option(False, "--max-tokens", help="Supports max_tokens parameter"),
    temperature: bool = typer.Option(False, "--temperature", help="Supports temperature parameter"),
    top_p: bool = typer.Option(False, "--top-p", help="Supports top_p parameter"),
    frequency_penalty: bool = typer.Option(False, "--frequency-penalty", help="Supports frequency_penalty parameter"),
    presence_penalty: bool = typer.Option(False, "--presence-penalty", help="Supports presence_penalty parameter"),
    stop: bool = typer.Option(False, "--stop", help="Supports stop parameter"),
    n: bool = typer.Option(False, "--n", help="Supports generating multiple completions"),
    stream: bool = typer.Option(False, "--stream", help="Supports streaming"),
    # Features
    tools: bool = typer.Option(False, "--tools", help="Supports tools/function calling"),
    vision: bool = typer.Option(False, "--vision", help="Supports vision/image input"),
    json_mode: bool = typer.Option(False, "--json-mode", help="Supports JSON mode"),
    system_prompt: bool = typer.Option(False, "--system-prompt", help="Supports system prompt"),
):
    """Add a new model."""
    # Create API parameters
    api_params = ApiParams(
        max_tokens=max_tokens,
        temperature=temperature,
        top_p=top_p,
        frequency_penalty=frequency_penalty,
        presence_penalty=presence_penalty,
        stop=stop,
        n=n,
        stream=stream,
    )

    # Create features
    features = Features(
        tools=tools,
        vision=vision,
        json_mode=json_mode,
        system_prompt=system_prompt,
    )

    # Create token costs if any cost parameters provided
    token_costs = None
    if any(x is not None for x in [input_cost, output_cost, context_window, training_cutoff]):
        token_costs = TokenCost(
            input_cost=input_cost or 0.0,
            output_cost=output_cost or 0.0,
            context_window=context_window,
            training_cutoff=training_cutoff,
        )

    # Create model capabilities
    capabilities = ModelCapabilities(
        model_id=model_id,
        providers=[provider],
        model_family=model_family,
        api_params=api_params,
        features=features,
        token_costs=token_costs,
    )

    # Load current data
    data = load_models()

    # Check if model already exists
    for model in data["models"].values():
        if model["model_id"] == model_id:
            if provider.value in model["providers"]:
                console.print(f"[red]Error: Model '{provider.value}/{model_id}' already exists.[/red]")
                raise typer.Exit(code=1)
            else:
                # Add provider to existing model
                model["providers"].append(provider.value)
                console.print(f"[yellow]Added provider '{provider.value}' to existing model '{model_id}'.[/yellow]")
                save_models(data)
                return

    # Add new model to models.json
    data["models"][model_id] = capabilities.model_dump()
    save_models(data)
    console.print(f"[green]Added new model '{model_id}' with provider '{provider.value}'.[/green]")


@app.command()
def delete(
    model_id: str = typer.Argument(help="Model identifier (e.g., 'gpt-4')"),
    provider: Provider | None = typer.Option(None, help="Provider to remove (if None, deletes entire model)"),
    force: bool = typer.Option(False, "--force", "-f", help="Force deletion without confirmation"),
):
    """Delete a model or remove a provider from a model."""
    # Load current data and validate model exists
    data, model_data = get_model_data(model_id)

    # If provider specified, only remove that provider
    if provider:
        validate_provider(model_data, provider, model_id)

        # Confirm deletion of provider
        if not force and not typer.confirm(f"Remove provider '{provider.value}' from model '{model_id}'?"):
            console.print("[yellow]Operation canceled.[/yellow]")
            return

        # Remove provider
        model_data["providers"].remove(provider.value)

        # If no providers left, delete the model
        if not model_data["providers"]:
            del data["models"][model_id]
            console.print(f"[green]Model '{model_id}' deleted (no providers remaining).[/green]")
        else:
            console.print(f"[green]Removed provider '{provider.value}' from model '{model_id}'.[/green]")
    else:
        # Confirm deletion of entire model
        if not force and not typer.confirm(f"Delete model '{model_id}'?"):
            console.print("[yellow]Operation canceled.[/yellow]")
            return

        # Delete model
        del data["models"][model_id]
        console.print(f"[green]Model '{model_id}' deleted.[/green]")

    # Save updated data
    save_models(data)


@app.command()
def update(
    model_id: str = typer.Argument(..., help="Model identifier (e.g., 'gpt-4')"),
    provider: Provider | None = typer.Option(None, help="Update specific provider"),
    model_family: str | None = typer.Option(None, help="Model family (e.g., 'GPT-4')"),
    input_cost: float | None = typer.Option(None, help="Input token cost per 1M tokens"),
    output_cost: float | None = typer.Option(None, help="Output token cost per 1M tokens"),
    cache_input_cost: float | None = typer.Option(None, help="Cached input token cost per 1M tokens"),
    cache_output_cost: float | None = typer.Option(None, help="Cached output token cost per 1M tokens"),
    context_window: int | None = typer.Option(None, help="Context window size in tokens"),
    training_cutoff: str | None = typer.Option(None, help="Training data cutoff date (e.g., '2024-01')"),
    # API Parameters
    max_tokens: bool | None = typer.Option(None, "--max-tokens", help="Supports max_tokens parameter"),
    temperature: bool | None = typer.Option(None, "--temperature", help="Supports temperature parameter"),
    top_p: bool | None = typer.Option(None, "--top-p", help="Supports top_p parameter"),
    frequency_penalty: bool | None = typer.Option(
        None, "--frequency-penalty", help="Supports frequency_penalty parameter"
    ),
    presence_penalty: bool | None = typer.Option(
        None, "--presence-penalty", help="Supports presence_penalty parameter"
    ),
    stop: bool | None = typer.Option(None, "--stop", help="Supports stop parameter"),
    n: bool | None = typer.Option(None, "--n", help="Supports generating multiple completions"),
    stream: bool | None = typer.Option(None, "--stream", help="Supports streaming"),
    # Features
    tools: bool | None = typer.Option(None, "--tools", help="Supports tools/function calling"),
    vision: bool | None = typer.Option(None, "--vision", help="Supports vision/image input"),
    json_mode: bool | None = typer.Option(None, "--json-mode", help="Supports JSON mode"),
    system_prompt: bool | None = typer.Option(None, "--system-prompt", help="Supports system prompt"),
):
    """Update a model's capabilities."""
    # Load current data and validate model exists
    data, model_data = get_model_data(model_id)

    # Update provider-specific fields
    validate_provider(model_data, provider, model_id)

    # Update fields if provided
    if model_family is not None:
        model_data["model_family"] = model_family

    # Update token costs
    if any(
        x is not None
        for x in [input_cost, output_cost, cache_input_cost, cache_output_cost, context_window, training_cutoff]
    ):
        if "token_costs" not in model_data:
            model_data["token_costs"] = {}
        if input_cost is not None:
            model_data["token_costs"]["input_cost"] = input_cost
        if output_cost is not None:
            model_data["token_costs"]["output_cost"] = output_cost
        if cache_input_cost is not None:
            model_data["token_costs"]["cache_input_cost"] = cache_input_cost
        if cache_output_cost is not None:
            model_data["token_costs"]["cache_output_cost"] = cache_output_cost
        if context_window is not None:
            model_data["token_costs"]["context_window"] = context_window
        if training_cutoff is not None:
            model_data["token_costs"]["training_cutoff"] = training_cutoff

    # Update API parameters
    api_params = [max_tokens, temperature, top_p, frequency_penalty, presence_penalty, stop, n, stream]
    if any(x is not None for x in api_params):
        if "api_params" not in model_data:
            model_data["api_params"] = {}
        if max_tokens is not None:
            model_data["api_params"]["max_tokens"] = max_tokens
        if temperature is not None:
            model_data["api_params"]["temperature"] = temperature
        if top_p is not None:
            model_data["api_params"]["top_p"] = top_p
        if frequency_penalty is not None:
            model_data["api_params"]["frequency_penalty"] = frequency_penalty
        if presence_penalty is not None:
            model_data["api_params"]["presence_penalty"] = presence_penalty
        if stop is not None:
            model_data["api_params"]["stop"] = stop
        if n is not None:
            model_data["api_params"]["n"] = n
        if stream is not None:
            model_data["api_params"]["stream"] = stream

    # Update features
    features = [tools, vision, json_mode, system_prompt]
    if any(x is not None for x in features):
        if "features" not in model_data:
            model_data["features"] = {}
        if tools is not None:
            model_data["features"]["tools"] = tools
        if vision is not None:
            model_data["features"]["vision"] = vision
        if json_mode is not None:
            model_data["features"]["json_mode"] = json_mode
        if system_prompt is not None:
            model_data["features"]["system_prompt"] = system_prompt

    # Save updated data
    save_models(data)
    console.print(f"[green]Updated model '{model_id}'.[/green]")


if __name__ == "__main__":
    app()



"""
LLM Registry - A tool for discovering, testing, and sharing LLM model capabilities.
"""

from .models import ModelCapabilities, Provider, TokenCost
from .registry import CapabilityRegistry
from .repository import CapabilityRepository
from .utils import create_model_capability

__version__ = "0.2.0"

__all__ = [
    "ModelCapabilities",
    "Provider",
    "TokenCost",
    "CapabilityRegistry",
    "CapabilityRepository",
    "create_model_capability",
]



"""
Registry for managing model capabilities data from both remote and local sources.
"""

import json
import time
from pathlib import Path
from typing import Dict, List

import httpx

from .models import ModelCapabilities, Provider

# Default remote URL
REMOTE_JSON_URL = "https://gist.githubusercontent.com/yamanahlawat/707a6ea05b079669347745a4484f5c6d/raw/models.json"


class CapabilityRegistry:
    """
    Registry for managing model capabilities data.
    """

    def __init__(
        self,
        remote_url: str = REMOTE_JSON_URL,
        cache_ttl: int = 3600,  # 1 hour
        offline_mode: bool = False,
        timeout: float = 10.0,  # 10 second timeout
    ):
        """
        Initialize the registry.
        Args:
            remote_url: URL to fetch remote JSON data from
            cache_ttl: Time in seconds to cache remote data
            offline_mode: If True, only use packaged data
            timeout: Timeout in seconds for HTTP requests
        """
        self.remote_url = remote_url
        self.cache_ttl = cache_ttl
        self.offline_mode = offline_mode
        self.timeout = timeout
        self._cache: Dict[str, ModelCapabilities] | None = None
        self._last_fetch: float | None = None
        self._client = httpx.Client(timeout=timeout)

    def get_models(self, provider: Provider | None = None) -> List[ModelCapabilities]:
        """
        Get all model capabilities, optionally filtered by provider.
        Args:
            provider: Optional provider to filter by
        Returns:
            List of ModelCapabilities objects
        """
        data = self._get_data()
        models = []

        for model_id, model_data in data["models"].items():
            if provider and provider not in model_data["providers"]:
                continue
            # Add model_id to data for validation
            model_cap = ModelCapabilities.model_validate({**model_data, "model_id": model_id})
            models.append(model_cap)

        return models

    def get_model(self, provider: Provider, model_id: str) -> ModelCapabilities | None:
        """
        Get capabilities for a specific model.
        Args:
            provider: The model provider
            model_id: The model identifier
        Returns:
            ModelCapabilities if found, None otherwise
        """
        data = self._get_data()

        if model_id not in data["models"]:
            return None

        model_data = data["models"][model_id]
        if provider.value not in model_data["providers"]:
            return None

        # Add model_id to data for validation
        return ModelCapabilities.model_validate({**model_data, "model_id": model_id})

    def _get_data(self) -> dict:
        """
        Get model data, trying remote first then falling back to packaged.
        """
        if not self.offline_mode:
            try:
                return self._get_remote_data()
            except (httpx.HTTPError, Exception):
                # Any failure, fall back to packaged data
                pass

        return self._get_packaged_data()

    def _get_remote_data(self) -> dict:
        """Get data from remote JSON."""
        now = time.time()

        # Use cache if valid
        if self._cache and self._last_fetch and (now - self._last_fetch < self.cache_ttl):
            return self._cache

        # Fetch fresh data
        response = self._client.get(self.remote_url)
        response.raise_for_status()

        self._cache = response.json()
        self._last_fetch = now
        return self._cache

    def _get_packaged_data(self) -> dict:
        """
        Get data from package JSON.
        """
        path = Path(__file__).parent / "data" / "models.json"
        with open(path) as f:
            return json.load(f)

    def __del__(self):
        """
        Ensure client is closed on cleanup.
        """
        if hasattr(self, "_client"):
            self._client.close()



"""
Utility functions for the LLM registry package.
"""

from .models import ApiParams, Features, ModelCapabilities, Provider, TokenCost


def create_model_capability(
    model_id: str,
    provider: Provider | str,
    model_family: str | None = None,
    supports_streaming: bool = False,
    supports_tools: bool = False,
    supports_vision: bool = False,
    supports_json_mode: bool = False,
    supports_system_prompt: bool = False,
    input_cost: float | None = None,
    output_cost: float | None = None,
    context_window: int | None = None,
    training_cutoff: str | None = None,
) -> ModelCapabilities:
    """
    Helper function to create a ModelCapabilities object with less verbose syntax.
    Args:
        model_id: Model identifier
        provider: Model provider
        model_family: Optional model family
        supports_streaming: Whether the model supports streaming
        supports_tools: Whether the model supports tools/function calling
        supports_vision: Whether the model supports vision inputs
        supports_json_mode: Whether the model supports JSON mode
        supports_system_prompt: Whether the model supports system prompts
        input_cost: Cost per 1M input tokens
        output_cost: Cost per 1M output tokens
        context_window: Context window size in tokens
        training_cutoff: Training data cutoff date
    Returns:
        ModelCapabilities object
    """
    # Convert string provider to enum if needed
    if isinstance(provider, str):
        try:
            provider = Provider(provider)
        except ValueError:
            provider = Provider.OTHER

    # Create token costs if applicable
    token_costs = None
    if input_cost is not None and output_cost is not None:
        token_costs = TokenCost(
            input_cost=input_cost,
            output_cost=output_cost,
            context_window=context_window,
            training_cutoff=training_cutoff,
        )

    # Create API params and features
    api_params = ApiParams(stream=supports_streaming)
    features = Features(
        vision=supports_vision,
        tools=supports_tools,
        json_mode=supports_json_mode,
        system_prompt=supports_system_prompt,
    )

    return ModelCapabilities(
        model_id=model_id,
        providers=[provider],
        model_family=model_family,
        api_params=api_params,
        features=features,
        token_costs=token_costs,
    )



"""Tests for the CLI module."""

import json
from pathlib import Path
from unittest.mock import mock_open, patch

import pytest
from typer.testing import CliRunner

from llm_registry.cli import app
from llm_registry.models import ApiParams, Features, ModelCapabilities, Provider, TokenCost


@pytest.fixture
def runner():
    """Create a CLI runner."""
    return CliRunner()


@pytest.fixture
def sample_model():
    """Create a sample model capability."""
    return ModelCapabilities(
        model_id="test-model",
        providers=[Provider.OPENAI],
        model_family="test",
        api_params=ApiParams(
            max_tokens=True,
            temperature=True,
            top_p=True,
            frequency_penalty=False,
            presence_penalty=False,
            stop=True,
            n=True,
            stream=True,
        ),
        features=Features(
            tools=True,
            vision=False,
            json_mode=True,
            system_prompt=True,
        ),
        token_costs=TokenCost(
            input_cost=0.1,
            output_cost=0.2,
            cache_input_cost=0.05,
            cache_output_cost=0.1,
            context_window=1000,
            training_cutoff="2024-01",
        ),
    )


@pytest.fixture
def mock_models_file(sample_model):
    """Create a mock models.json file."""
    return {
        "models": {
            "test-model": sample_model.model_dump(),
        }
    }


def test_list_models(runner, sample_model):
    """Test listing models command."""
    with patch("llm_registry.cli.CapabilityRegistry") as mock_registry:
        mock_registry.return_value.get_models.return_value = [sample_model]
        result = runner.invoke(app, ["list"])
        assert result.exit_code == 0
        # Check basic info
        assert "test-model" in result.stdout
        assert "openai" in result.stdout
        assert "test" in result.stdout  # model family
        # Check costs (just basic presence)
        assert "$0.1" in result.stdout  # Input cost
        assert "$0.2" in result.stdout  # Output cost


def test_list_models_by_provider(runner, sample_model):
    """Test listing models filtered by provider."""
    with patch("llm_registry.cli.CapabilityRegistry") as mock_registry:
        mock_registry.return_value.get_models.return_value = [sample_model]
        result = runner.invoke(app, ["list", "--provider", "openai"])
        assert result.exit_code == 0
        assert "test-model" in result.stdout

        # Test non-existent provider
        mock_registry.return_value.get_models.return_value = []
        result = runner.invoke(app, ["list", "--provider", "anthropic"])
        assert result.exit_code == 0
        assert "No models found" in result.stdout


def test_add_model(runner, mock_models_file):
    """Test adding a new model."""
    mock_file = mock_open(read_data=json.dumps(mock_models_file))
    with patch("builtins.open", mock_file), patch("llm_registry.cli.get_models_file") as mock_path:
        mock_path.return_value = Path("test.json")
        result = runner.invoke(
            app,
            [
                "add",
                "new-model",
                "--provider",
                "openai",
                "--model-family",
                "test",
                "--input-cost",
                "0.1",
                "--output-cost",
                "0.2",
                "--context-window",
                "1000",
                "--training-cutoff",
                "2024-01",
                "--stream",
                "--tools",
                "--json-mode",
                "--system-prompt",
            ],
        )
        assert result.exit_code == 0
        assert "Added new model 'new-model'" in result.stdout


def test_add_existing_model_provider(runner, mock_models_file):
    """Test adding a provider to an existing model."""
    mock_file = mock_open(read_data=json.dumps(mock_models_file))
    with patch("builtins.open", mock_file), patch("llm_registry.cli.get_models_file") as mock_path:
        mock_path.return_value = Path("test.json")
        result = runner.invoke(
            app,
            [
                "add",
                "test-model",
                "--provider",
                "anthropic",
            ],
        )
        assert result.exit_code == 0
        assert "Added provider 'anthropic' to existing model 'test-model'" in result.stdout


def test_delete_model(runner, mock_models_file):
    """Test deleting a model."""
    mock_file = mock_open(read_data=json.dumps(mock_models_file))
    with patch("builtins.open", mock_file), patch("llm_registry.cli.get_models_file") as mock_path:
        mock_path.return_value = Path("test.json")

        # Test deleting entire model with force flag
        result = runner.invoke(app, ["delete", "test-model", "-f"])
        assert result.exit_code == 0
        assert "Model 'test-model' deleted" in result.stdout

        # Test deleting non-existent model
        result = runner.invoke(app, ["delete", "non-existent", "-f"])
        assert result.exit_code == 1
        assert "not found" in result.stdout


def test_delete_provider(runner, mock_models_file):
    """Test removing a provider from a model."""
    mock_file = mock_open(read_data=json.dumps(mock_models_file))
    with patch("builtins.open", mock_file), patch("llm_registry.cli.get_models_file") as mock_path:
        mock_path.return_value = Path("test.json")

        # Test removing existing provider
        result = runner.invoke(app, ["delete", "test-model", "--provider", "openai", "-f"])
        assert result.exit_code == 0
        assert "deleted (no providers remaining)" in result.stdout

        # Test removing non-existent provider
        result = runner.invoke(app, ["delete", "test-model", "--provider", "anthropic", "-f"])
        assert result.exit_code == 1
        assert "not found" in result.stdout


def test_update_model(runner, mock_models_file):
    """Test updating a model's capabilities."""
    mock_file = mock_open(read_data=json.dumps(mock_models_file))
    with patch("builtins.open", mock_file), patch("llm_registry.cli.get_models_file") as mock_path:
        mock_path.return_value = Path("test.json")

        # Test updating various fields
        result = runner.invoke(
            app,
            [
                "update",
                "test-model",
                "--model-family",
                "new-family",
                "--input-cost",
                "0.15",
                "--cache-input-cost",
                "0.075",
                "--context-window",
                "2000",
                "--vision",
            ],
        )
        assert result.exit_code == 0
        assert "Updated model 'test-model'" in result.stdout

        # Test updating non-existent model
        result = runner.invoke(app, ["update", "non-existent"])
        assert result.exit_code == 1
        assert "not found" in result.stdout


def test_update_provider_specific(runner, mock_models_file):
    """Test updating provider-specific capabilities."""
    mock_file = mock_open(read_data=json.dumps(mock_models_file))
    with patch("builtins.open", mock_file), patch("llm_registry.cli.get_models_file") as mock_path:
        mock_path.return_value = Path("test.json")

        # Test updating with non-existent provider
        result = runner.invoke(
            app,
            [
                "update",
                "test-model",
                "--provider",
                "anthropic",
                "--input-cost",
                "0.15",
            ],
        )
        assert result.exit_code == 1
        assert "not found" in result.stdout



"""Tests for the CapabilityRepository class."""

import pytest

from llm_registry import CapabilityRepository, Provider
from llm_registry.utils import create_model_capability


@pytest.fixture
def temp_repo(tmp_path):
    """Create a temporary repository."""
    return CapabilityRepository(tmp_path)


@pytest.fixture
def sample_model():
    """Create a sample model capability."""
    return create_model_capability(
        model_id="test-model",
        provider=Provider.OPENAI,  # This will be converted to providers list internally
        model_family="test",
        supports_streaming=True,
        supports_tools=True,
        input_cost=0.1,
        output_cost=0.2,
        context_window=1000,
    )


def test_save_and_get_model(temp_repo, sample_model):
    """Test saving and retrieving a model."""
    # Save model
    file_path = temp_repo.save_model_capabilities(sample_model)
    assert file_path.exists()

    # Get model back
    retrieved = temp_repo.get_model_capabilities(sample_model.providers[0], sample_model.model_id)
    assert retrieved is not None
    assert retrieved.model_id == sample_model.model_id
    assert Provider.OPENAI in retrieved.providers
    assert retrieved.token_costs.input_cost == sample_model.token_costs.input_cost


def test_list_models(temp_repo, sample_model):
    """Test listing models."""
    # Save model
    temp_repo.save_model_capabilities(sample_model)

    # List all models
    models = temp_repo.list_models()
    assert len(models) == 1
    assert models[0].model_id == sample_model.model_id

    # List by provider
    models = temp_repo.list_models(provider=sample_model.providers[0])
    assert len(models) == 1
    models = temp_repo.list_models(provider=Provider.ANTHROPIC)
    assert len(models) == 0


def test_delete_model(temp_repo, sample_model):
    """Test deleting a model."""
    # Save model
    temp_repo.save_model_capabilities(sample_model)

    # Delete model
    success = temp_repo.delete_model(sample_model.providers[0], sample_model.model_id)
    assert success is True

    # Verify deletion
    model = temp_repo.get_model_capabilities(sample_model.providers[0], sample_model.model_id)
    assert model is None

    # Try deleting non-existent model
    success = temp_repo.delete_model(Provider.ANTHROPIC, "non-existent")
    assert success is False


def test_overwrite_model(temp_repo, sample_model):
    """Test overwriting an existing model."""
    # Save model
    temp_repo.save_model_capabilities(sample_model)

    # Modify and save again
    modified_model = create_model_capability(
        model_id=sample_model.model_id,
        provider=sample_model.providers[0],  # Use the same provider
        input_cost=0.2,  # Changed cost
        output_cost=0.4,
    )
    temp_repo.save_model_capabilities(modified_model)

    # Verify changes
    retrieved = temp_repo.get_model_capabilities(modified_model.providers[0], modified_model.model_id)
    assert retrieved.token_costs.input_cost == 0.2
    assert retrieved.token_costs.output_cost == 0.4



"""
Repository for storing and retrieving model capabilities.
"""

import json
import os
from pathlib import Path

from .models import ModelCapabilities, Provider

# Default paths
DEFAULT_DATA_DIR = Path.home() / ".llm-registry"
DEFAULT_MODELS_DIR = DEFAULT_DATA_DIR / "models"


class CapabilityRepository:
    """
    Repository for storing and retrieving model capabilities.
    """

    def __init__(self, data_dir: Path | None = None):
        """
        Initialize the repository with a data directory.
        Args:
            data_dir: Directory to store model capability data.
                     If None, defaults to ~/.llm-registry
        """
        if data_dir is None:
            self.data_dir = DEFAULT_MODELS_DIR
        else:
            self.data_dir = data_dir / "models"

        self.data_dir.mkdir(parents=True, exist_ok=True)

        # Cache for loaded capabilities
        self._capabilities_cache: dict[str, ModelCapabilities] = {}

    def _get_model_path(self, provider: Provider, model_id: str) -> Path:
        """
        Get the file path for a model's capability data.
        Args:
            provider: The model provider
            model_id: The model identifier
        Returns:
            Path to the model's JSON file
        """
        provider_value = provider.value if isinstance(provider, Provider) else provider
        return self.data_dir / f"{provider_value}_{model_id}.json"

    def save_model_capabilities(self, capabilities: ModelCapabilities) -> Path:
        """
        Save model capabilities to the repository.
        Args:
            capabilities: The capabilities to save
        Returns:
            Path where the capabilities were saved
        """
        # Save a copy for each provider
        paths = []
        for provider in capabilities.providers:
            file_path = self._get_model_path(provider, capabilities.model_id)
            with open(file_path, "w", encoding="utf-8") as f:
                f.write(capabilities.model_dump_json(indent=2))

            # Update cache
            cache_key = f"{provider.value}_{capabilities.model_id}"
            self._capabilities_cache[cache_key] = capabilities
            paths.append(file_path)

        return paths[0]  # Return the first path for backward compatibility

    def get_model_capabilities(self, provider: Provider, model_id: str) -> ModelCapabilities | None:
        """
        Get capabilities for a specific model.
        Args:
            provider: The model provider
            model_id: The model identifier
        Returns:
            ModelCapabilities if found, None otherwise
        """
        provider_value = provider.value if isinstance(provider, Provider) else provider
        cache_key = f"{provider_value}_{model_id}"

        # Check cache first
        if cache_key in self._capabilities_cache:
            cap = self._capabilities_cache[cache_key]
            if provider in cap.providers:
                return cap

        # Check file system
        file_path = self._get_model_path(provider, model_id)
        if not file_path.exists():
            return None

        try:
            with open(file_path, "r", encoding="utf-8") as f:
                data = json.load(f)
                capabilities = ModelCapabilities.model_validate(data)

                # Verify provider is in the model's providers
                if provider not in capabilities.providers:
                    return None

                # Update cache
                self._capabilities_cache[cache_key] = capabilities
                return capabilities
        except (json.JSONDecodeError, Exception) as e:
            print(f"Error loading model capabilities: {e}")
            return None

    def list_models(self, provider: Provider | None = None) -> list[ModelCapabilities]:
        """
        List all models in the repository, optionally filtering by provider.
        Args:
            provider: Optional provider to filter by
        Returns:
            List of ModelCapabilities objects
        """
        result = []
        seen_models = set()  # Track unique model IDs

        # Filter pattern for filenames
        pattern = f"{provider.value}_*.json" if provider else "*.json"

        for file_path in self.data_dir.glob(pattern):
            try:
                with open(file_path, "r", encoding="utf-8") as f:
                    data = json.load(f)
                    capabilities = ModelCapabilities.model_validate(data)

                    # Skip if we've already seen this model
                    if capabilities.model_id in seen_models:
                        continue

                    # If provider specified, check if it's in the model's providers
                    if provider and provider not in capabilities.providers:
                        continue

                    result.append(capabilities)
                    seen_models.add(capabilities.model_id)

                    # Update cache for each provider
                    for p in capabilities.providers:
                        cache_key = f"{p.value}_{capabilities.model_id}"
                        self._capabilities_cache[cache_key] = capabilities
            except Exception as e:
                print(f"Error loading {file_path}: {e}")

        return result

    def delete_model(self, provider: Provider, model_id: str) -> bool:
        """
        Delete a model from the repository.
        Args:
            provider: The model provider
            model_id: The model identifier
        Returns:
            True if deleted, False if not found
        """
        file_path = self._get_model_path(provider, model_id)
        if not file_path.exists():
            return False

        try:
            os.remove(file_path)

            # Remove from cache
            provider_value = provider.value if isinstance(provider, Provider) else provider
            cache_key = f"{provider_value}_{model_id}"
            if cache_key in self._capabilities_cache:
                del self._capabilities_cache[cache_key]

            return True
        except Exception as e:
            print(f"Error deleting model: {e}")
            return False
